"use strict";
/**
 * @license
 * Copyright 2019 Ludan Stoecklé
 * SPDX-License-Identifier: Apache-2.0
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.LanguageSynFrench = void 0;
const LanguageSyn_1 = require("./LanguageSyn");
const stopwordsFr = require("stopwords-fr");
const frenchStemmer = require("snowball-stemmer.jsx/dest/french-stemmer.common.js");
class LanguageSynFrench extends LanguageSyn_1.LanguageSyn {
    constructor() {
        super();
        this.iso2 = 'fr';
        this.stemmer = new frenchStemmer.FrenchStemmer();
    }
    getStandardStopWords() {
        return stopwordsFr;
    }
    extractWords(input) {
        let res = super.extractWords(input);
        // we just leave [Pp]uisqu [Jj]usqu [Ll]orsqu as they are
        const regexp = new RegExp("^(D|d|Q|q|L|l|S|s|J|j|T|t|M|m|N|n)'", 'g');
        res = res.map((elt) => {
            return elt.replace(regexp, '');
        });
        // sometimes it results in having empty elements
        res = res.filter((elt) => elt.length > 0);
        return res;
    }
}
exports.LanguageSynFrench = LanguageSynFrench;
//# sourceMappingURL=LanguageSynFrench.js.map